# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

import sys
import os

import re
import tempfile
import commands

class Header:
    def __init__(self, path, include=None):
        self.path = path
        self.include = path
        if include:
            self.include = include

    def get_symbol_names(self):
        names = []
        handle = open(self.path, "r")
        matcher = re.compile('#define\s*(\S*)')
        for line in handle.readlines():
            match = matcher.search(line)
            if match:
                names.append(match.expand("\\1"))
        return names

    def expand_symbol(self, name):
        return self.expand_symbol_compile(name)
    
    def expand_symbol_compile(self, name):
        result = None

        dir = tempfile.mkdtemp(prefix='defines.')
        os.system("cp %s %s" % (self.include, dir))

        for format in ("%d", "%s", "%f"):
            (fd, codefile) = tempfile.mkstemp(suffix='.c',dir=dir)
            os.write(fd, '#include "%s"\n' % self.include)
            os.write(fd, '#include <stdio.h>\n')
            os.write(fd, 'int main(void) {\n')
            os.write(fd, '  printf("%s", %s);\n' % (format, name))
            os.write(fd, '  return 0;\n')
            os.write(fd, '}\n')
            os.close(fd)

            binary = os.path.join(dir, 'a.out')
            command = "gcc -o %s -Wall -Werror %s 2> /dev/null" % (
                binary, codefile)
            status = os.system(command)
            if os.WIFSIGNALED(status):
                raise Exception, "subprocess signalled"
            if os.WIFEXITED(status) and os.WEXITSTATUS(status) == 0:
                command = binary
                (status, output) = commands.getstatusoutput(command)
                if os.WIFSIGNALED(status):
                    raise Exception, "subprocess signalled"
                if os.WIFEXITED(status) and os.WEXITSTATUS(status) == 0:
                    if format == "%s":
                        result = '"' + output  + '"'
                    # we don't use %x anymore, we can't make it work well
                    elif format == "0x%x":
                        # FutureWarning for 2.4 values > sys.maxint, so
                        # make long
                        result = output  + 'L' 
                    else:
                        result = output


        os.system("rm -rf %s" % dir)
        return result
        
if __name__ == '__main__':
    header = Header(sys.argv[1])
    names = header.get_symbol_names()
    for name in names:
        print "%s = %s" % (name, header.expand_symbol(name))
    
            
