# -*- Mode: Python -*-
# vi:si:et:sw=4:sts=4:ts=4

import os
import sys
import struct
from fcntl import ioctl

import defines
import structs

class Device:
    def __init__(self, path):
        self.path = path
        self._vcap = None
        self._vpic = None
        self._fd = -1

        self._fd = os.open(self.path, os.O_RDWR)

    def __del__(self):
        self.close()

    def close(self):
        if self._fd != -1:
            os.close(self._fd)
            self._fd = -1

    def query(self):
        if not self._vcap:
            self._vcap = structs.object_from_ioctl(structs.VideoCapability,
                self._fd, defines.VIDIOCGCAP)
        return self._vcap

    def query_video_picture(self):
        if not self._vpic:
            self._vpic = structs.object_from_ioctl(structs.VideoPicture,
                self._fd, defines.VIDIOCGPICT)
        return self._vpic

    def get_channels(self):
        if not self._vcap:
            self.query()

        channels = []
        for i in range(0, self._vcap.channels):
            buffer = struct.pack('i', i)
            channels.append(structs.object_from_ioctl(structs.VideoChannel,
                self._fd, defines.VIDIOCGCHAN, buffer=buffer))
        return channels

    def set_channel(self, number):
        buffer = struct.pack('i', 0)
        ioctl(self._fd, defines.VIDIOCGCHAN, buffer)
